
# q-expansion of newform 1232.4.a.c, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1232
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-4], [-12], [-7], [-11], [-10], [24], [94], [180], [30], [94], [-214], [-48], [-8], [-30], [54], [-360], [-178], [292], [-312], [728], [1288], [-66], [-390], [-1510], [90], [-302], [-240], [-166], [1662], [-1544], [-2358], [1206], [-2630], [-126], [1456], [992], [-3188], [252], [-3882], [3300], [-124], [-1860], [-1318], [-1266], [-830], [4408], [4246], [-1710], [-2044], [54], [4416], [-808], [-5400], [3882], [7536], [1596], [-20], [-5374], [-786], [-3878], [-8226], [-3926], [-618], [686], [2094], [-2444], [-7378], [6840], [-5614], [9066], [5256], [2350], [-4306], [14380], [-10746], [-6870], [15728], [12846], [-5740], [-2292], [-10834], [11400], [-15958], [268], [15324], [2238], [-9046], [11898], [-6512], [-11676], [-16920], [5188], [-21036], [2284], [11004], [-5244], [-1902], [8194], [10406], [-20900], [10038], [8154], [-2730], [-3692], [-3946], [3312], [-10920], [25800], [-24700], [9016], [-1618], [-11310], [24700], [2500], [3450], [-18416], [7014], [-5070], [-30600], [22364], [-31522], [-30210], [-12492], [-21608], [28266], [5978], [-6330], [8710], [-2506], [-17060], [-22392], [16828], [10514], [12084], [1940], [-10800], [-11378], [15528], [5082], [-24086], [6210], [-20612], [3948], [-5692], [29070], [20582], [-36156], [-19784], [34068], [-19510], [-24474], [-49388], [30216], [4492], [-28932], [32056], [-22182], [29792], [30702], [-5244], [-24150], [-15512], [-52404], [-5070], [-18702], [-40496], [-23686], [13082], [4164], [-29736], [37280], [54696], [-19042], [-2222], [14406], [-36356], [-38790], [-4124], [-47782], [17728], [-8310], [-65806], [-53772], [-38928], [6972], [2486], [44548], [-6634], [19932], [-59170], [20592], [-46604], [-54456], [79044], [-38598], [74030], [-63934], [-75744], [-57660], [-49638], [-62828], [-81520], [29720], [-22236], [-37260], [-76718], [-30348], [72906], [3652], [-59002], [-16896], [-7688], [21960], [60468], [23198], [-56216], [52668], [25440], [7626], [38870], [-14780], [-2346], [-9560], [80178], [65234], [-2898], [-16368], [-52148], [54984], [105482], [-53426], [-23408], [87522], [46492], [-75684], [-70006], [75138], [3204], [76698], [-18708], [38422], [-6110], [-30130], [-51756], [11580], [23122], [26472], [87532], [1248], [92774], [64200], [61200], [107258], [-69162], [71184], [22298], [-18230], [27876], [-75760], [-48560], [30084], [110750], [-87550], [9126], [41512], [-9666], [-89994], [47716], [-52002], [121592], [-6788], [77558], [130576], [48656], [-79562], [56652], [-47578], [41858], [-3378], [98232], [-27752], [123912], [12506], [68326], [-143028], [-100378], [-84054], [-142718], [107796], [-63714], [-27204], [85086], [-21018], [-101734], [28938], [103258], [-96636], [29706], [-80054], [-147868], [116298], [110032], [11172], [700], [-82930], [36372], [86156], [-100776], [-72526], [-56448], [145002], [110286], [-8228], [148152], [-84448], [-41064], [115200], [-42634], [-97170], [178678], [85514], [128910], [-130724], [94506], [-130930], [139228], [-135884], [-79008], [-19470], [72950], [110370], [-189308], [25050], [-202844], [-45156], [53498], [-147564], [-7678], [-4376], [107486], [73074], [-3138], [32092], [172554], [-9684], [-181204], [-126812], [70416], [213336], [81898], [-57274], [142230], [-116984], [-140422], [-140388], [11634], [-105414], [34890], [-96432], [-186262], [63828], [-136656], [30948], [-62396], [-222478], [-26514], [-36440], [-193390], [180372], [-119684], [70368], [-69090], [-138488], [128006], [-106764], [-165048], [-202732], [-261504], [151640], [-212952], [-70278], [-98696], [78978], [214018], [-159642], [-186932], [9956], [70744], [-75870], [211274], [49152], [156396], [239200], [24480], [133526], [167992], [240846], [44356], [-29214], [197102], [-207342], [-67736], [-212754], [-159018], [230788], [116606], [-186846], [172732], [32412], [100352], [-34218], [-51012], [144340], [-26026], [81528], [-223224], [129406], [49734], [-91176], [-56958], [-10666], [-290664], [179214], [309062], [-88896], [-113892], [272454], [-126524], [96936]]
